/*
    ---------------------------------------------------------------------- copyright and license ---

    file: src/app.hpp

    Copyright © 2014, 2016—2018 Van de Bugger.

    This file is a part of Tapper.

    Tapper is free software: you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Tapper is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License along with Tapper.  If not,
    see <http://www.gnu.org/licenses/>.

    SPDX-License-Identifier: GPL-3.0-or-later

    ---------------------------------------------------------------------- copyright and license ---
*/

/**
    @file
    Application class interface.
**/

#ifndef _APP_HPP_
#define _APP_HPP_

#include "base.hpp"

#include "x.hpp"
#include "switcher.hpp"

class app_t : public object_t {

    public :

        class error_t : public std::runtime_error {
            public :
                explicit error_t( string_t const & what ) : std::runtime_error( what ) {};
        }; // class error_t

    public :

        app_t( int argc, char * argv[] );

    private :

        typedef x::kb_t::desc_t::key_code_range_t key_code_range_t;
        typedef layout_switcher_t::map_t          map_t;
        typedef layout_switcher_t::mode_t         mode_t;

        enum opt_key_t {
            ok_list_keys    = 1000,
            ok_list_layouts,
            ok_xkb,
            ok_gnome,
            ok_bell,
        }; // enum opt_key_t

        static mode_t    default_mode();
        static ::error_t parse_opt_or_arg( int key, char * _arg, struct argp_state * state );

        void   parse_argv( int argc, char * argv[] );
        void   parse_arg( string_t const & arg );
        uint_t parse_key( string_t const & str );
        uint_t parse_layout( string_t const & layout );
        uint_t parse_int( string_t const & str, string_t const & prefix );

        void init();
        void list_keys();
        void list_layouts();
        void show_map();

        key_code_range_t const & key_code_range();
        strings_t const &        key_names();
        strings_t const &        layout_names();

        string_t key_name( uint_t code );
        string_t layout_name( uint_t code );

    private :

        char const * const  prog_name;

        x::display_p        display;
        x::kb_p             kb;

        key_code_range_t    _key_code_range;
        strings_t           _key_names;
        strings_t           _layout_names;

        map_t               map;
        mode_t              mode;
        bool                bell = false;

}; // class app_t

#endif // _APP_HPP_

// end of file //
