/*
    ---------------------------------------------------------------------- copyright and license ---

    file: src/gnome.hpp

    Copyright © 2014, 2016—2018 Van de Bugger.

    This file is a part of Tapper.

    Tapper is free software: you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Tapper is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License along with Tapper.  If not,
    see <http://www.gnu.org/licenses/>.

    SPDX-License-Identifier: GPL-3.0-or-later

    ---------------------------------------------------------------------- copyright and license ---
*/

/**
    @file
    Gnome wrappers interface.
**/

#ifndef _GNOME_HPP_
#define _GNOME_HPP_

#include "config.h"

#if ENABLE_GNOME

#include "base.hpp"

#include <gio/gio.h>

namespace gnome {

    uint_t const layout_max = 256;

    // ---------------------------------------------------------------------------------------------
    // error_t
    // ---------------------------------------------------------------------------------------------

    class error_t : public std::runtime_error {
        public :
            typedef error_t            myself_t;
            typedef std::runtime_error parent_t;
            error_t( string_t const & message, GError * error = nullptr );
            error_t( myself_t const & other );
            virtual ~error_t();
            myself_t & operator =( myself_t const & other ) = delete;
            bool matches( GQuark domain, gint code ) const;
        private :
            GError * _error;
    }; // class error_t

    template< typename value_t >
    struct deleter_t {
        void operator ()( value_t * ptr ) const {
            g_object_unref( ptr );
        };
    };

    template<>
    struct deleter_t< GVariant > {
        void operator ()( GVariant * ptr ) const {
            g_variant_unref( ptr );
        };
    };

    typedef std::unique_ptr< GDBusProxy, deleter_t< GDBusProxy > >  bus_p;
    typedef std::unique_ptr< GVariant, deleter_t< GVariant > >      variant_p;

    // ---------------------------------------------------------------------------------------------
    // input_source_manager_t
    // ---------------------------------------------------------------------------------------------

    class input_source_manager_t : public object_t {

        public:

            input_source_manager_t();
            ~input_source_manager_t();

            strings_t   keyboard_layouts();
            void        activate_layout( uint_t idx, bool bell = false );

        private:

            void     _reset();
            string_t _eval( string_t const & jscript );

        private:

            bus_p       _bus;

    }; // class input_source_manager_t

    typedef ptr_t< input_source_manager_t >  input_source_manager_p;

}; // namespace gnome

#endif // ENABLE_GNOME

#endif // _GNOME_HPP_

// end of file //
