/*
    ---------------------------------------------------------------------- copyright and license ---

    file: src/switcher.cpp

    Copyright © 2014, 2016—2018 Van de Bugger.

    This file is a part of Tapper.

    Tapper is free software: you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Tapper is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License along with Tapper.  If not,
    see <http://www.gnu.org/licenses/>.

    SPDX-License-Identifier: GPL-3.0-or-later

    ---------------------------------------------------------------------- copyright and license ---
*/

#include "switcher.hpp"

#include <cassert>

// -------------------------------------------------------------------------------------------------
// layout_switcher_t
// -------------------------------------------------------------------------------------------------

layout_switcher_t::error_t::error_t(
    string_t const & msg
) :
    std::runtime_error( msg )
{
}; // class error_t

layout_switcher_t::layout_switcher_t(
    x::kb_t &       kb,
    map_t const &   map,
    mode_t          mode,
    bool            bell
) :
    tapper_t( kb ),
    _map( map ),
    _mode( mode ),
    _bell( bell )
{
    if ( _mode == sm_gnome ) {
        #if ENABLE_GNOME
            _ism.reset( new gnome::input_source_manager_t );
        #else
            ERR( "The program is built without Gnome support" );
        #endif // ENABLE_GNOME
    }; // if
}; // ctor

void
layout_switcher_t::on_tap(
    KeyCode key_code
) {
    map_t::const_iterator it = _map.find( key_code );
    if ( it != _map.end() ) {
        switch ( _mode ) {
            case sm_xkb : {
                _kb.lock_group( it->second );
                if ( _bell ) {
                    _kb.bell();
                }; // if
                _kb.display().flush();
            } break;
            #if ENABLE_GNOME
                case sm_gnome : {
                    _ism->activate_layout( it->second, _bell );
                } break;
            #endif // ENABLE_GNOME
            default : {
                assert( 0 );
            } break;
        }; // switch
    }; // if
}; // on_tap

std::ostream &
operator <<(
    std::ostream &              stream,
    layout_switcher_t::mode_t   mode
) {
    switch ( mode ) {
        case layout_switcher_t::sm_xkb : {
            stream << "Xkb";
        } break;
        case layout_switcher_t::sm_gnome : {
            stream << "Gnome";
        } break;
        default : {
            assert( 0 );
            stream << "(* unknown mode *)";
        } break;
    };
    return stream;
};

// end of file //
