/*
    ---------------------------------------------------------------------- copyright and license ---

    file: src/switcher.hpp

    Copyright © 2014, 2016—2018 Van de Bugger.

    This file is a part of Tapper.

    Tapper is free software: you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Tapper is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License along with Tapper.  If not,
    see <http://www.gnu.org/licenses/>.

    SPDX-License-Identifier: GPL-3.0-or-later

    ---------------------------------------------------------------------- copyright and license ---
*/

#ifndef _SWITCHER_HPP_
#define _SWITCHER_HPP_

#include "base.hpp"

#include "tapper.hpp"
#include "x.hpp"
#include "gnome.hpp"

#include <map>

// -------------------------------------------------------------------------------------------------
// layout_switcher_t
// -------------------------------------------------------------------------------------------------

class layout_switcher_t : public tapper_t {

    public :

        /** KeyCode => layout index map. **/
        typedef std::map< KeyCode, uint_t > map_t;
        enum mode_t {
            sm_xkb,
            sm_gnome
        }; // enum mode_t

        class error_t : public std::runtime_error {
            public :
                explicit error_t( string_t const & msg );
        }; // class error_t

        layout_switcher_t( x::kb_t & kb, map_t const & map, mode_t mode, bool bell = false );

    protected :

        virtual void on_tap( KeyCode key_code );

    private :

        map_t   _map;
        mode_t  _mode;
        bool    _bell;
        #if ENABLE_GNOME
            gnome::input_source_manager_p   _ism;
        #endif // ENABLE_GNOME

}; // class layout_switcher_t

std::ostream &
operator <<(
    std::ostream &              stream,
    layout_switcher_t::mode_t   mode
);

#endif // _SWITCHER_HPP_

// end of file //
